/*
Yahoo-Transport auth.c: handles authorizing a session, and subscriptions
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <yahootrans.h>

/* 
function to take a presence packet with the encoded user/pass and log them in
*/
int yt_auth_user(jpacket jp)
{
    char *username;
    char *password;
    xmlnode res;
    session s;

    if(jp==NULL||jp->to->resource==NULL)
    {
        xmlnode_free(jp->x);
        return 0;
    }
    res = (xmlnode)jid_xres(jp->to);
    if(res==NULL)
    {
        xmlnode_free(jp->x);
        return 0;
    }
    username = xmlnode_get_attrib(res,"username");
    password = xmlnode_get_attrib(res,"pass");
    if(username==NULL||password==NULL) 
    {
        log_warn(ZONE,"(%s) has a bad resource",jid_full(jp->from));
        xmlnode_free(jp->x);
        return 1;
    }
    s=yt_session_find_by_jid(jp->from);
    if(s==NULL)
    {
        thread t;
        log_warn(ZONE,"new session:[%s](%s)",jid_full(jp->from),username);
        t=yt_session_create(username,password,jp,0);
        if(t!=NULL) s=t->data;
    }
    if(s!=NULL) 
        yt_send_yahoo_status(s,jp->x);
    else
        xmlnode_free(jp->x);
    return 0;
}

/*
take a jabber:iq:register and subscribe to them, and start a new user session 
*/
int yt_user_reg(jpacket jp)
{
    xmlnode query;
    session s;
    char *username,*password;

    if(jp==NULL) 
    {
        xmlnode_free(jp->x);
        return;
    }
    s=yt_session_find_by_jid(jp->from);
    log_debug(ZONE,"[reg] recieved a registration from: %s",jid_full(jp->from));
    query=xmlnode_get_tag(jp->x,"query");
    if(query==NULL) 
    {
        xmlnode_free(jp->x);
        return;
    }
    if(s!=NULL&&xmlnode_get_tag(query,"remove")!=NULL)
    {
        log_warn(ZONE,"unsubscribing");
        yt_send_presence(s,jp,jid_full(s->id),"unsubscribe","Removing Subscription",NULL);
        yt_session_end(s);
        return;
    }
    username=xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query,"username")));
    password=xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query,"password")));
    if(username==NULL||password==NULL)
    {
        log_debug(ZONE,"[reg] Bad Registration");
        yt_senderror(jp->from,"406","Bad Password and/or Username!");
        xmlnode_free(jp->x);
        return;
    }
    if(s!=NULL)
    {
        log_warn(ZONE,"reg for existing user!");
        /* change registration */
        /* unsubscribe from user */
        yt_send_presence(s,NULL,jid_full(s->id),"unsubscribe","Changing Subscription",NULL);
        /* Kill current session */
        session_unlink(s);
        if(s->stream!=NULL) tstream_close(s->stream);
        pth_sleep(1);
    }
    log_debug(ZONE,"[reg] attempting to log in %s as user: %s",jid_full(jp->from),username);
    yt_session_create(username,password,jp,1);
    jutil_iqresult(jp->x);
    jpacket_reset(jp);
    ehandler_send(ehandler_yt,jp->x,jp->to->server);
    return 0;
}

/*
take a username and pass, and return a jid encoding
and send a subscribe packet to that jid
*/
jid yt_subscribe_auth(jid to,char*user,char*pass)
{
    jid ret;
    xmlnode x;
    spool sp;
    session s;
    if(to==NULL||user==NULL||pass==NULL) return;
    sp=spool_new(to->p);
    s=yt_session_find_by_jid(to);
    log_debug(ZONE,"[sub] Subscribing to %s",to);
    spooler(sp,yt__host,"/registered?username=",user,"&pass=",pass,sp);
    yt_send_presence(s,NULL,spool_print(sp),"subscribe","Setting Subscription",NULL);
    ret=jid_new(to->p,spool_print(sp));
    return ret;
}

/* 
subscribe to a yahoo user
*/
void yt_subscribe_user(jpacket jp)
{
    session s;
        
    if(jp==NULL) return;
    s=yt_session_find_by_jid(jp->from);
    if(s==NULL||s->Con==NULL) 
    {
        yt_senderror(jp->from,"407","Server failed to find your session");
        xmlnode_free(jp->x);
        return;
    }
    log_debug(ZONE,"[sub] User %s subscribing to %s",jid_full(jp->from),jp->to->user);
    if(jp->to->user==NULL)
    { /* subscribing to the transport */
        yt_send_presence(s,jp,jid_full(jp->to),"subscribed","Transport Subscribe",NULL);
    }
    else if(s->connected==0)
    {
        yt_send_presence(s,jp,jid_full(jp->to),"unsubscribed","Not Connected to Yahoo!",NULL);
    }
    else
    {
        yt_send_presence(s,NULL,jid_full(jp->to),"subscribed","Transport Subscribe",NULL);
        log_debug(ZONE,"[sub] Subscriptions sent, adding buddy %s",jp->to->user);
        yahoo_add_buddy(s->Con,jp->from->user,s->Con->login_id,"jabber_yt","");
        yt_send_presence(s,jp,jid_full(jp->to),"subscribe","Transport Subscribe",NULL);
    }
}

/* 
unsubscribe from a yahoo user
*/
void yt_unsubscribe_user(jpacket jp)
{
    session s;

    if(jp==NULL) return;
    s=yt_session_find_by_jid(jp->from);
    if(s==NULL||jp->to->user==NULL||s->Con==NULL||s->Con->login_id==NULL) 
    {
        yt_senderror(jp->from,"407","Server failed to find your session");
        xmlnode_free(jp->x);
        return;
    }
    log_debug(ZONE,"[unsub] User %s unsub to %s",jid_full(jp->from),jp->to->user);
    yahoo_remove_buddy(s->Con,jp->to->user,s->Con->login_id,"jabber_yt","");
    yt_send_presence(s,jp,jid_full(jp->to),"unsubscribe","Transport Unsubscribe",NULL);
}

/*
strip the resource from the jid
*/
void yt_strip_jid(jid j)
{
    if(j==NULL) return;
    jid_set(j,NULL,JID_RESOURCE);
}

/*
perform a res insensitive compare
*/
int j_cmp_nores(jid a,jid b)
{
    jid ja,jb;
    int ret;
    if(a==NULL||b==NULL) return -1;
    ja=jid_new(a->p,jid_full(a));
    jb=jid_new(b->p,jid_full(b));
    jid_set(ja,NULL,JID_RESOURCE);
    jid_set(jb,NULL,JID_RESOURCE);
    ret=jid_cmp(ja,jb);
    return ret;
}

/*
updated the status for the session based on the presece xmlnode x
*/
void yt_send_yahoo_status(session s, xmlnode x)
{
    pool p;
    char *show = NULL;
    xmlnode xshow;
    if(s==NULL||x==NULL||s->Con==NULL) 
    {
        if(x!=NULL) xmlnode_free(x);
        return;
    }
    p=xmlnode_pool(x);
    log_debug(ZONE,"[auth] updating status");
    /* XXX user already online, update his/her status */
    if(s->connected==0) 
    {
        log_debug(ZONE,"[auth] tried to update status, but not connected yet");
        xmlnode_free(x);
        return;
    }
    xshow=xmlnode_get_tag(x,"show");
    if(xshow!=NULL)
    show=pstrdup(p,xmlnode_get_data(xshow));
    log_debug(ZONE,"[auth] (%s) switching status (%d) -> (%s)",jid_full(s->j),s->status,show);
    if(show==NULL||j_strcmp(show,"chat")==0)
    {
        if(s->status!=YAHOO_STATUS_AVAILABLE)
        yahoo_cmd_set_back_mode(s->Con,YAHOO_STATUS_AVAILABLE,xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x,"status"))));
        s->status = YAHOO_STATUS_AVAILABLE;
    }
    else if(j_strcmp(show,"away")==0)
    {
        yahoo_cmd_set_away_mode(s->Con,YAHOO_STATUS_BRB,xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x,"status"))));
        s->status = YAHOO_STATUS_BRB;
    }
    else if(j_strcmp(show,"xa")==0)
    {
        yahoo_cmd_set_away_mode(s->Con,YAHOO_STATUS_IDLE,xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x,"status"))));
        s->status = YAHOO_STATUS_IDLE;
    }
    else if(j_strcmp(show,"dnd")==0)
    {
        yahoo_cmd_set_away_mode(s->Con,YAHOO_STATUS_BUSY,xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x,"status"))));
        s->status = YAHOO_STATUS_BUSY;
    }
    jutil_tofrom(x);
    ehandler_send(ehandler_yt,x,s->sendto->server);
}
