/*
Yahoo-Transport config.c Reads the config XML file
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <yahootrans.h>

xmlnode yahoo_trans_config = NULL;
char *etherx__host=NULL;
char *etherx__secret=NULL;
int etherx__port=5269;
char *yt__host=NULL;

/*
handle the config.. load all values into the vars
*/
int handle_config(xmlnode conf)
{
    /* XXX This is where we handle config
     * 0 = good handle
     * 1 = error handling
     */
    xmlnode next;

    log_debug(ZONE,"[init] Handling Config Data");

    /* we got the chunk from the config file, 
    make sure we init everything depending on config reguardless if they were configured  */
    next=xmlnode_get_firstchild(conf);
    while(next!= NULL)
    {
        if(j_strcmp(xmlnode_get_name(next),"etherx")==0)
        {
            xmlnode etherx=xmlnode_get_firstchild(next);
            while(etherx!=NULL)
            {
                if(j_strcmp(xmlnode_get_name(etherx),"host")==0)
                {
                    log_debug(ZONE,"[init] Using etherx host: %s",xmlnode_get_data(etherx));
                    etherx__host=pstrdup(main_pool,xmlnode_get_data(etherx));
                }
                else if(j_strcmp(xmlnode_get_name(etherx),"secret")==0)
                {
                    log_debug(ZONE,"[init] Using etherx secret: %s",xmlnode_get_data(etherx));
                    etherx__secret=pstrdup(main_pool,xmlnode_get_data(etherx));
                 }
                 etherx=xmlnode_get_nextsibling(etherx);
             }
        }
        else if(j_strcmp(xmlnode_get_name(next),"name")==0)
        {
	        log_debug(ZONE,"[init] using alias: %s\n",xmlnode_get_data(next));
            yt__host=pstrdup(main_pool,xmlnode_get_data(next));
	    }
	    next=xmlnode_get_nextsibling(next);
    }
    if(etherx__host==NULL) 
    {
        log_error(ZONE,"[CONFIG] Unable to find Etherx host in config, using '127.0.0.1'");
        etherx__host=pstrdup(main_pool,"127.0.0.1");
    }
    if(etherx__secret==NULL) 
    {
        log_error(ZONE,"[CONFIG] Unable to find Etherx secret in config, using 'test'");
        etherx__secret=pstrdup(main_pool,"test");
    }
    if(yt__host==NULL) 
    {
        log_error(ZONE,"[CONFIG] Unable to find Yahoo host name in config");
        exit(1);
    }
    return 0;
}


/*
load the config file and handle it
*/
int load_config(char *configfile) {
    if(configfile==NULL) configfile=YAHOOTRANSCONF;
    yahoo_trans_config=xmlnode_file(configfile);
    if(yahoo_trans_config!=NULL) log_warn(ZONE,"[init] reading config from (%s)",configfile);
    handle_config(yahoo_trans_config);
    return 0;
}
