/*
Yahoo-Transport external.c connects and talks with etherx (incoming)
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <yahootrans.h>

ehandler ehandler_yt;

/*
init the transport (etherx)
*/
void yt_init()
{
    log_warn(ZONE,"[init] Transport (%s) was told to start up",yt__host);
    xmlnode_free(yahoo_trans_config);
    if(etherx__port!=5269)
        ehandler_yt=ehandler_new_ex(&yt_etherx_receive, yt__host,"jabber:component:accept",etherx__port);
    else
        ehandler_yt=ehandler_new(&yt_etherx_receive, yt__host,"jabber:server");
    if(ehandler_yt==NULL)
    {
        log_error(ZONE,"[init] Failed to connect with etherx");
        exit(1);
    }
    return;
}

void yt_etherx_receive(epacket e)
{
    jpacket jp;

    if(e==NULL) return;
    log_debug(ZONE,"[etherx] %s: %s",yt__host,xmlnode2str(e->x));
    switch(e->type)
    {
        case EPACKET_NORMAL:
            xmlnode_hide_attrib(e->x,"etherx:to");
            xmlnode_hide_attrib(e->x,"etherx:from");
            jp=jpacket_new(e->x);
            yt_parse_packet(jp);
            log_debug(ZONE,"[etherx] returning from incoming packet");
            break;
        case EPACKET_ERROR:
            log_debug(ZONE,"[etherx] packet error: %s\n",xmlnode2str(e->x));
            break;
        default:
            xmlnode_free(e->x);
    }
    return;
}

char *str_to_UTF8(pool p,unsigned char *in)
{
    int n,i=0;
    char *result = NULL;
    /* for now, allocate more than enough space... */
    if(in==NULL) return NULL;
    result = (char*)pmalloco(p,strlen(in)*2+1);
    for(n=0;n<strlen(in);n++)
    {
        char *font;
        long c=(long)in[n];
        /*check for <font ....> html codes*/
        font=strstr(&in[n],"<font ");
        if(strlen(&in[n])>7&&strncasecmp(&in[n],"<font ",6)==0)
        {
            font=strchr(font,'>');
            if(font!=NULL)
            {
                n=((long)font)-((long)in);
                continue;
            }
        }
        /*strip out color codes*/
        if(c==27)
        {  
            n+=2;
            if(in[n]=='x')
                n++;
            if(in[n]=='3')
                n++;
            n++;
            continue;
        }
        if(c==0x0D||c==0x0a)
            continue;
        if(c<128)
        {
            result[i++]=(char)c;
        }
        else
        {
            result[i++]=(char)((c>>6)|192);
            result[i++]=(char)((c&63)|128);
        }
    }
    result[i]='\0';
    return result;
}

unsigned char *UTF8_to_str(pool p,unsigned char *in)
{
    int n,i;
    unsigned char *result = NULL;
    if(in==NULL) return NULL;
    result=(unsigned char*)pmalloco(p,strlen(in));
    n = 0;
    i = 0;
    while(n<=strlen(in)-1)
    {
       long c=(long)in[n];
       if(c<0x80)
           result[i++]=(char)c;
       else
       {
           if((c&0xC0)==0xC0)
               result[i++]=(char)(((c&0x03)<<6)|(((long)in[++n])&0x3F));
           else if((c&0xE0)==0xE0)
               result[i++]=(char)(((c&0xF)<<4)|(((long)in[++n])&0x3F)|(((long)in[++n])&0x3F)); 
           else if((c&0xF0)==0xF0)
               n+=3;
           else if((c&0xF8)==0xF8)
               n+=4;
           else if((c&0xFC)==0xFC)
               n+=5;
        }
        n++;
    }
        result[i]='\0';
        return result;
}

void yt_send_presence(session s,jpacket jp,char *from,char *type,char *status,char *show)
{
    xmlnode presence;
    jid j;
    if(s==NULL||s->exit_flag>0||from==NULL) return;
    presence=xmlnode_new_tag("presence");
    xmlnode_put_attrib(presence,"to",jid_full(s->sendto));
    xmlnode_put_attrib(presence,"from",from);
    if(type!=NULL)
    {
        xmlnode_put_attrib(presence,"type",type);
    }
    else
    {
        /* insert it into the presence list */
        j=jid_new(xmlnode_pool(presence),from);
        if(j->user!=NULL)
        {
            int found_match=0;
            while(s->online_users!=NULL)
            {
                if(jid_cmp(s->online_users->j,j)==0)
                {
                    found_match=1;
                    break;
                }
                else
                {
                    s->online_users=s->online_users->next;
                }
            }
            if(!found_match) /* add to the list */
            {
                xmlnode temp=xmlnode_dup(presence);
                user_list new=pmalloco(xmlnode_pool(temp),sizeof(_user_list));
                new->p=xmlnode_pool(temp);
                new->x=temp;
                new->j=jid_new(xmlnode_pool(temp),xmlnode_get_attrib(temp,"from"));
            }
        }
    }
    if(status!=NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(presence,"status"),status,-1);
    if(show!=NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(presence,"show"),show,-1);
    ehandler_send(ehandler_yt,presence,s->sendto->server);
    if(jp!=NULL) xmlnode_free(jp->x);
}


void yt_send_message(session s,jpacket jp,char *from,char *type,char *body,char *subject)
{
    xmlnode message;
    if(s==NULL||s->exit_flag>0||from==NULL||body==NULL) return;
    message=xmlnode_new_tag("message");
    if(type!=NULL)
        xmlnode_put_attrib(message,"type",type);
    if(subject!=NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(message,"subject"),subject,-1);
    xmlnode_put_attrib(message,"from",from);
    xmlnode_put_attrib(message,"to",jid_full(s->sendto));
    xmlnode_insert_cdata(xmlnode_insert_tag(message,"body"),body,-1);
    ehandler_send(ehandler_yt,message,s->sendto->server);
    if(jp!=NULL) xmlnode_free(jp->x);
}

/*
build and send a jabber error
*/
void yt_senderror(jid j,char *code,char *errmsg)
{
    xmlnode err,error;
    if(j==NULL||code==NULL||errmsg==NULL) return;
    err=xmlnode_new_tag("message");
    xmlnode_put_attrib(err,"to",jid_full(j));
    xmlnode_put_attrib(err,"from",yt__host);
    xmlnode_put_attrib(err,"type","error");
    error=xmlnode_insert_tag(err,"error");
    xmlnode_put_attrib(error,"code",code);
    xmlnode_insert_cdata(error,errmsg,-1);
    ehandler_send(ehandler_yt,err,j->server);
}
