/*
Yahoo-Transport main.c Main loop for yahoo transport
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <libxode.h>
#include <yahootrans.h>

int y_debug;
ppdb PPDB;
ppdb y_ppdb;
pool main_pool;

int main(int argc, char *argv[])
{
    pth_event_t sigevt;
    sigset_t set;
    char *config=NULL,*c;
    int help,i,sig,sz;

    main_pool=pool_new();
    y_debug=0;
    help=0;
    for(i=1;i<argc;i++)
    {
        c=argv[i]+1;
        switch(*c)
        {
        case 'D':
            etherx_debug();
            break;
        case 's':
            /* etherx secret config */
            etherx__secret=pstrdup(main_pool,argv[++i]);
            log_warn(ZONE,"option %c: %s",*c,argv[i]);
            break;
        case 'e':
            /* etherx host config */
            etherx__host=pstrdup(main_pool,argv[++i]);
            log_warn(ZONE,"option %c: %s",*c,argv[i]);
            break;
        case 'p':
            /* etherx port config */
            etherx__port=j_atoi(argv[++i],5269);
            log_warn(ZONE,"option %c: %s",*c,argv[i]);
            break;
        case 'n':
            /* host name config */
            yt__host=pstrdup(main_pool,argv[++i]);
            log_warn(ZONE,"option %c: %s",*c,argv[i]);
            break;
        case 'c':
            config=pstrdup(main_pool,argv[++i]);
            log_warn(ZONE,"option %c: %s",*c,argv[i]);
            break;
        default:
            log_warn(ZONE,"invalid option %c",*c);
            help=1;
        }
    }
    if(help)
    {
        printf("Usage: yahootrans [options]\n\tcmdline options\n\t---------------\n\n\t-n yahoo_transport_hostname\n\t-e etherx_host\n\t-s etherx_secret\n\t-p etherx_port\n\t-D (Enable debug output)\n\t-c config_file\n");
        exit(0);
    }
    /* initialize libjabber and app.* globals */
    log_init(LOGSTDERR,NULL);
    if(load_config(config))
    {
        log_error(ZONE,"[init] Failed to Load config.");
        pth_kill();
        return 1;
    }
    log_debug(ZONE,"[loading] transport starting up...[%s][%s]",etherx__host,etherx__secret);

    /* intitialize libetherx and tell it to connect to etherx */
    etherx_init(etherx__secret, etherx__host);
    yt_init();
    sigemptyset(&set);
    sigaddset(&set,SIGHUP);
    sigaddset(&set,SIGINT);
    sigaddset(&set,SIGTERM);
    sigaddset(&set,SIGKILL);
    pth_sigmask(SIG_UNBLOCK,&set, NULL);
    sigevt = pth_event(PTH_EVENT_SIGS,&set,&sig);
    pth_wait(sigevt);
    yt_trans_stop();
    pool_free(main_pool);
    log_error(ZONE,"[exiting] Recieved KILL, shutting down.");
    ppdb_free(PPDB);
    ppdb_free(y_ppdb);
    pth_sleep(3);
    pth_kill();
    return 0;
}


