/*
Yahoo-Transport parser.c Handles data from etherx
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <yahootrans.h>

/*
parse an incoming jabber message
*/
int yt_parse_message(jpacket jp)
{
    char *body;
    session s;
    xmlnode tmpx;
    if(jp==NULL) return;
            
    s=yt_session_find_by_jid(jp->from);
    if(s==NULL)
    {
        yt_senderror(jp->from,"407","Yahoo! Server doesn't think you are connected.");
        log_debug(ZONE,"[message] No connection found for jid: %s",jid_full(jp->from));
    }
    else if(jp->to->user==NULL) 
    {  /* possible place for admin mod_announce type thing? */
        yt_senderror(jp->from,"406","Message sent to yahoo with no user.");
        log_debug(ZONE,"[message] Message sent with no user!");
    }
    else
    {
        tmpx=xmlnode_get_tag(jp->x,"body");
        if(tmpx==NULL) return;
        body=UTF8_to_str(jp->p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
        if(body!=NULL&&s->Con!=NULL&&s->Con->login_id!=NULL) 
            yahoo_cmd_msg_offline(s->Con,s->Con->login_id,jp->to->user,body);
        else
            yt_senderror(jp->from,"406","failed to send message");    
    }
    xmlnode_free(jp->x);
    return 0;
}

/* 
incoming register get
*/
int yt_parse_iq(jpacket jp)
{
    session s;
    char *from,*to;
    xmlnode query;
    if(jp==NULL) return;

    s=yt_session_find_by_jid(jp->from);
    jutil_iqresult(jp->x);
    query=xmlnode_get_tag(jp->x,"query");
    if(query==NULL) query=xmlnode_insert_tag(jp->x,"query");
    xmlnode_put_attrib(query,"xmlns","jabber:iq:register");
    if(s==NULL||s->Con==NULL) 
    {
        xmlnode_insert_tag(query,"username");
        xmlnode_insert_tag(query,"password");
        xmlnode_insert_cdata(xmlnode_insert_tag(query,"key"),jutil_regkey(NULL,jid_full(jp->to)),-1);
        xmlnode_insert_cdata(xmlnode_insert_tag(query,"instructions"),"Enter your YAHOO! Messenger Username and Password.",-1);
    }
    else 
    {
        xmlnode_insert_tag(query,"registered");
        xmlnode_insert_tag(query,"password");
        xmlnode_insert_cdata(xmlnode_insert_tag(query,"key"),jutil_regkey(NULL,jid_full(jp->to)),-1);
        xmlnode_insert_cdata(xmlnode_insert_tag(query,"username"),s->Con->user,-1);
        xmlnode_insert_cdata(xmlnode_insert_tag(query,"instructions"),"Enter your YAHOO! Messenger Username and Password.",-1);
    }
    jpacket_reset(jp);
    ehandler_send(ehandler_yt,jp->x,jp->to->server);
    return 0;
}

/* 
incoming packets from etherx get parsed here
*/
int yt_parse_packet(jpacket jp)
{
    xmlnode temp;
    session s;
    jid j;
        
    if(jp==NULL) return 1;
    switch(jp->type)
    {
    case JPACKET_IQ:
        if(jpacket_subtype(jp)==JPACKET__GET)
        {
            if(NSCHECK(jp->iq,NS_REGISTER))
                yt_parse_iq(jp);
            else if(NSCHECK(jp->x,NS_AGENTS))
                groupchat_agent_request(jp);
            else if(NSCHECK(jp->iq,NS_VERSION))
            {
                xmlnode query;
                jp->x=jutil_iqresult(jp->x);
                query=xmlnode_get_tag(jp->x,"query");
                if(query==NULL)query=xmlnode_insert_tag(jp->x,"query");
                xmlnode_put_attrib(query,"xmlns","jabber:iq:version");
                xmlnode_insert_cdata(xmlnode_insert_tag(query,"name"),"YAHOO! Messenger",-1);
                jpacket_reset(jp);
                ehandler_send(ehandler_yt,jp->x,jp->to->server);
            }
            else if(NSCHECK(jp->iq,NS_TIME))
            {
                xmlnode query;
                jp->x=jutil_iqresult(jp->x);
                query=xmlnode_get_tag(jp->x,"query");
                if(query==NULL)query=xmlnode_insert_tag(jp->x,"query");
                xmlnode_put_attrib(query,"xmlns","jabber:iq:time");
                xmlnode_insert_cdata(xmlnode_insert_tag(query,"utc"),jutil_timestamp(),-1);
                xmlnode_insert_cdata(xmlnode_insert_tag(query,"tz"),"gmt",-1);
                jpacket_reset(jp);
                ehandler_send(ehandler_yt,jp->x,jp->to->server);
            }
            else
            {
                xmlnode_free(jp->x);
                break;
            }
        } 
        else /* type='set' */
        {
            if(NSCHECK(jp->iq,NS_REGISTER))
                yt_user_reg(jp);
            else if(NSCHECK(jp->iq,NS_OOB))
            {
                char *url,*desc;
                spool sp;
                session s;
                s=yt_session_find_by_jid(jp->from);
                if(s==NULL||s->Con==NULL)
                    break;
                url=xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(xmlnode_get_tag(jp->x,"query"),"url")));
                desc=xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(xmlnode_get_tag(jp->x,"query"),"desc")));
                log_debug(ZONE,"[YAHOO] sendind url (%s) to user (%s) activeID (%s)",url,jp->to->user,s->Con->user);
                sp=spool_new(jp->p);
                spooler(sp,s->Con->user,", (who is using jabber), would like to send you a file.  This file (",desc,") is located at: ",url," ... Due to the limitations of Jabber and Yahoo, this file could not be sent directly, but will be implemented at a later date.  Sorry for the inconvenience",sp);
                log_debug(ZONE,"[FILE XFER] sending content: %s",spool_print(sp));
                yahoo_cmd_msg_offline(s->Con, s->Con->login_id,jp->to->user, spool_print(sp));
                xmlnode_free(jp->x);
            }
            else
                xmlnode_free(jp->x);
        }
        break;
    case JPACKET_MESSAGE:
        if(jpacket_subtype(jp)==JPACKET__ERROR)
            xmlnode_free(jp->x);
        else
            yt_parse_message(jp);
        break;
    case JPACKET_S10N:
        switch(jpacket_subtype(jp))
        {
        case JPACKET__SUBSCRIBE:
            yt_subscribe_user( jp );
            break;
        case JPACKET__UNSUBSCRIBE:
            yt_unsubscribe_user( jp );
            break;
        default:
            xmlnode_free(jp->x);
            break;
        }
        break;
    case JPACKET_PRESENCE:
        switch(jpacket_subtype(jp))
        {
        case JPACKET__PROBE:
            j=jid_new(jp->p,jid_full(jp->to));
            yt_strip_jid(j);
            temp=ppdb_primary(y_ppdb,j);
            if(temp==NULL)
            {
                log_debug(ZONE,"[PROBE] Have no info for this probe");
            }
            else
            {
                xmlnode send = xmlnode_dup(temp);
                log_debug(ZONE,"[PROBE] Returning probe request");
                yt_strip_jid(jp->from);
                xmlnode_put_attrib( send, "to", jid_full(jp->from));
                xmlnode_put_attrib(send,"from",jid_full(jp->to));
                ehandler_send(ehandler_yt,send,jp->from->server);
            }
            xmlnode_free(jp->x);
            break;
        case JPACKET__UNAVAILABLE:
            if(jp->to->resource==NULL||jp->to->user!=NULL)
            { /* presence to a user */
                jutil_tofrom(jp->x);
                jpacket_reset(jp);
                ehandler_send(ehandler_yt,jp->x,jp->to->server);
                break;
            }
            PPDB=ppdb_insert(PPDB,jp->from,jp->x);
            j=jid_new(jp->p,jid_full(jp->from));
            yt_strip_jid(j);
            temp=xmlnode_dup(ppdb_primary(PPDB,j));
            s=yt_session_find_by_jid(jp->from);
            if(temp!=NULL) /*session is still active, update status*/
            {
                if(s!=NULL)
                    yt_send_yahoo_status(s,temp);
            }
            else /*all sessions closed, exit session*/
            {
                if(s!=NULL) 
                {
                    /* send presence to all users */
                    while(s->online_users!=NULL)
                    {
                        jpacket new;
                        user_list u=s->online_users;
                        s->online_users=s->online_users->next;
                        jutil_tofrom(u->x);
                        xmlnode_put_attrib(u->x,"type","unavailable");
                        xmlnode_hide(xmlnode_get_firstattrib(u->x));
                        new=jpacket_new(u->x);
                        ehandler_send(ehandler_yt,u->x,new->to->server);
                    }
                    yt_session_end(s);
                }
            }
            jutil_tofrom(jp->x);
            jpacket_reset(jp);
            ehandler_send(ehandler_yt,jp->x,jp->to->server);
            log_debug(ZONE,ZONE);
            break;
        case JPACKET__AVAILABLE:
            if(jp->to->resource==NULL)
            {
                user_list new;
                /* avail to a user, add it to our user list */
                s=yt_session_find_by_jid(jp->from);
                if(s==NULL)
                { /* we don't have a session yet -- XXX queue*/
                    xmlnode_free(jp->x);
                    break;
                }
                new=pmalloco(jp->p,sizeof(_user_list));
                new->p=jp->p;
                new->j=jp->to;
                new->x=jp->x;
                new->next=s->online_users;
                s->online_users=new;
                break;
            }
            PPDB=ppdb_insert(PPDB,jp->from,jp->x);
            yt_auth_user(jp);
            break;
        default:
            xmlnode_free(jp->x);
            break;
        };
        break;
    default:
        xmlnode_free(jp->x);
    }
    return 0;
}

