/*
Yahoo-Transport sessions.c Handles the main session loop
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <yahootrans.h>
#define YAHOO_PAGER_HOST "cs.yahoo.com"
#define YAHOO_PAGER_PORT        5050

session sessions=NULL;
struct yahoo_options Opts={0,NULL,0};

/*
cleans up the mess when the socket dies
*/
void yt_session_cleanup(void *arg)
{
    session s;    
    if(arg==NULL) return;
    s=(session)arg;
    s->stream=NULL;
    s->exit_flag=1;
    s->Con=NULL;
}

/* 
unlink a session from the list (gets freed later)
*/
void session_unlink(session s)
{
    log_debug(ZONE,"unlinking session (%p)",s);    
    if(s==NULL) return;
    if(s==sessions) sessions=s->next;
    if(s->prev!=NULL)
        s->prev->next=s->next;
    if(s->next!=NULL)
        s->next->prev=s->prev;

    /* unlink all conferences 
    while(s->conferences!=NULL)
    {
        pool_free(s->conferences->p);
        s->conferences=s->conferences->next;
    }
    */
    log_debug(ZONE,"[session] Unlinked Session (%s)",jid_full(s->j));
}

/*
check to see if we should exit
returning non-zero will end the thread
*/
int yt_session_isdone(void *arg)
{
    session s=(session)arg;
    return s->exit_flag;
}	

/* 
send an exit flag to the session 
*/
void yt_session_end(session s)
{
    if(s==NULL)
        log_warn(ZONE,"NULL Session told to end!");
    else
    {
        log_debug(ZONE, "Session (%s) told to end.",jid_full(s->j));
        session_unlink(s);
        s->exit_flag=1;
    }
}

/*
return a ssession, given a jid
*/
session yt_session_find_by_jid(jid j)
{
    session next;
    if(j==NULL) return NULL;
    log_debug(ZONE,"[session] finding session for (%s)",jid_full(j));
    if(sessions==NULL) return NULL;
    for(next=sessions;next!=NULL;next=next->next)
        if(j_cmp_nores(j,next->j)==0)
            return next;
    return NULL;
}

/*
status packet from a yahoo user to a jabber user
*/
void process_packet_status(struct yahoo_packet *pkt,session s)
{
    int i;

    log_debug(ZONE,"[status] processing status");
    if(s->Con==NULL) return;
    if(s==NULL||pkt==NULL||s->Con->user==NULL) 
        return;
    if (pkt->service==YAHOO_SERVICE_LOGOFF&&pkt->msgtype==0xFFFFFFFF)
    {
        yt_senderror(s->sendto,"502","Error from Yahoo! server, You have been disconnected, Send Online presence again to re-log in.");
        session_unlink(s);
        s->exit_flag=1;
        return;
    }
    log_debug(ZONE,"[status] other user presence. (%d)",pkt->idstatus_count);
    for(i=0;i<pkt->idstatus_count;i++)
    {
        struct yahoo_idstatus *rec;
        spool id;
        pool p;
        
        p=pool_new();
        id=spool_new(p);

        rec=pkt->idstatus[i];
        spooler(id,rec->id,"@",yt__host,id);
        log_debug(ZONE,"[status] ID: %s, in_pager: %d",rec->id,rec->in_pager);
        if(rec->in_pager)
        {
            if(pkt->service==YAHOO_SERVICE_ISAWAY)
            {
                switch(rec->status)
                {
                case YAHOO_STATUS_ONPHONE:
                case YAHOO_STATUS_IDLE:
                case YAHOO_STATUS_STEPPEDOUT:
                case YAHOO_STATUS_OUTTOLUNCH:
                case YAHOO_STATUS_BRB:
                    yt_send_presence(s,NULL,spool_print(id),NULL,yahoo_get_status_string(rec->status),"away");
                    break;
                case YAHOO_STATUS_NOTATDESK:
                case YAHOO_STATUS_NOTINOFFICE:
                case YAHOO_STATUS_ONVACATION:
                case YAHOO_STATUS_NOTATHOME:
                    yt_send_presence(s,NULL,spool_print(id),NULL,yahoo_get_status_string(rec->status),"xa");
                    break;
                case YAHOO_STATUS_BUSY:
                    yt_send_presence(s,NULL,spool_print(id),NULL,yahoo_get_status_string(rec->status),"dnd");
                    break;
                default:
                    yt_send_presence(s,NULL,spool_print(id),NULL,"Yahoo! Online",NULL);
                    break;
                }
                log_debug(ZONE,"[DEBUG] %d ",rec->status);
                
            }
            else
            {
                yt_send_presence(s,NULL,spool_print(id),NULL,"Yahoo! Online",NULL);
            }
        }
        else
        {
            yt_send_presence(s,NULL,spool_print(id),"unavailable","Yahoo! user went offline",NULL);
        }
        log_debug(ZONE,"IDStatus ID (%d) = %s",i,spool_print(id));
        pool_free(p);
    }
}

/*
handles a message from yahoo to jabber
*/
void process_packet_message(pool p,struct yahoo_packet *pkt,session s)
{
    log_debug(ZONE,"[message] processing message");
    if(pkt->msg!=NULL)
    {
        spool id=spool_new(p);
        spooler(id,pkt->msg_id,"@",yt__host,id);
        if(pkt->msgtype==YAHOO_MSGTYPE_BOUNCE) 
            yt_send_message(s,NULL,spool_print(id),NULL,"Message not sent: user went offline",NULL);
        else
        {
            char *msg=str_to_UTF8(p,pkt->msg);
            log_debug(ZONE,"Converting (%s) to (%s)",pkt->msg,msg);
            yt_send_message(s,NULL,spool_print(id),"chat",msg,NULL);
        }
    }
}

void process_packet_calendar(struct yahoo_packet *pkt,session s)
{

}

void process_packet_chatinvite(struct yahoo_packet *pkt,session s)
{

}

void process_packet_newmail(struct yahoo_packet *pkt,session s)
{

}

void process_packet_ping(struct yahoo_packet *pkt,session s)
{

}

/*
handles (sortof) a filetrnasfer request from yahoo to jabber
*/
void process_packet_fileXfer(struct yahoo_packet *pkt,session s)
{
    pool p=pool_new();
    spool sp=spool_new(p);
    spool sq=spool_new(p);;
    spooler(sp,pkt->file_from,"@",yt__host,sp);
    spooler(sq,"The Yahoo user, ",pkt->file_from,
        "would like to send you a file, due to limitations on yahoo-t, please point your browser to: ",
        pkt->file_url," for this file, Description follows: ",pkt->file_description,sq);
    yt_send_message(s,NULL,spool_print(sp),NULL,spool_print(sq),NULL);
    pool_free(p);
}

void handle_err(thread t, tstream_buffer buffer)
{
    tstream ts;
    session s;
    if(t==NULL)return;
    ts=(tstream)t->data;
    if(ts==NULL)return;
    s=(session)ts->data;
    if(s==NULL)return;
    log_debug(ZONE,"handle_err(%p,%s)",t,buffer->data);
    session_unlink(s);
    s->exit_flag=1;
}

void handle_read(thread t, void *buffer, int bufsz)
{
    struct yahoo_rawpacket *rawpkt;
    struct yahoo_packet *pkt;
    tstream ts;
    session s;
    pool p;

    if(t==NULL) return;
    ts=(tstream)t->data;
    if(ts==NULL) return;
    s=(session)ts->data;
    if(s==NULL) return;
    if(s->Con==NULL) return;
    if(buffer==NULL)
    {
        /*init*/
        pool_cleanup(s->stream->p,&yt_session_cleanup,(void*)s);    
        return;
    }
    p=pool_new();
    yahoo_addtobuffer(s->Con,buffer,bufsz);
    if(s->Con->io_buf_curlen>102)
    {
        while((rawpkt=yahoo_getpacket(s->Con)))
        {
            log_debug(ZONE,"TOP OF PACKET LOOP");
            pkt=yahoo_parsepacket(s->Con,rawpkt);
            log_debug(ZONE,"Received packet:");
            log_debug(ZONE,"Service = %s",yahoo_get_service_string(pkt->service));
            log_debug(ZONE,"Real ID = %s",pkt->real_id);
            log_debug(ZONE,"Active ID = %s",pkt->active_id);
            log_debug(ZONE,"Connection ID = %X",pkt->connection_id);
            log_debug(ZONE,"Magic ID = %X",pkt->magic_id);
            log_debug(ZONE,"Unknown Flag 1 = %X",pkt->unknown1);
            log_debug(ZONE,"Message Type = %X",pkt->msgtype);
            log_debug(ZONE,"Raw Content = %s",rawpkt->content);
            switch(pkt->service)
            {
            case YAHOO_SERVICE_USERSTAT:
            case YAHOO_SERVICE_CHATLOGON:
            case YAHOO_SERVICE_CHATLOGOFF:
            case YAHOO_SERVICE_LOGON:
            case YAHOO_SERVICE_LOGOFF:
            case YAHOO_SERVICE_ISAWAY:
            case YAHOO_SERVICE_ISBACK:
                process_packet_status(pkt,s);
                break;
            case YAHOO_SERVICE_MESSAGE:
            case YAHOO_SERVICE_CHATMSG:
            case YAHOO_SERVICE_SYSMESSAGE:
                process_packet_message(p,pkt,s);
                break;
            case YAHOO_SERVICE_CALENDAR:
                process_packet_calendar(pkt,s);
                break;
            case YAHOO_SERVICE_CHATINVITE:
                process_packet_chatinvite(pkt,s);
                break;
            case YAHOO_SERVICE_NEWMAIL:
            case YAHOO_SERVICE_NEWPERSONALMAIL:
                process_packet_newmail(pkt,s);
                break;
            case YAHOO_SERVICE_NEWCONTACT:
                if(pkt->msg)
                    process_packet_message(p,pkt,s);
                else
                    process_packet_status(pkt,s);
                break;
            case YAHOO_SERVICE_GROUPRENAME:
                yahoo_get_config(s->Con);
                break;
            case YAHOO_SERVICE_CONFINVITE:
            case YAHOO_SERVICE_CONFLOGON:
            case YAHOO_SERVICE_CONFLOGOFF:
            case YAHOO_SERVICE_CONFMSG:
               break;
            case YAHOO_SERVICE_PING:
               process_packet_ping(pkt,s);
               break;
            case YAHOO_SERVICE_FILETRANSFER:
               process_packet_fileXfer(pkt,s);
               break;
            }
            yahoo_free_packet(pkt);
            yahoo_free_rawpacket(rawpkt);
            log_debug(ZONE,"BOTTON OF PACKET LOOP");
        }
    }
    pool_free(p);
}

/*
main session loop (thread)
*/
void yt_session_main(void *arg)
{
    thread t;
    session s;
    pth_event_t done;
    if(arg==NULL) return;
    t=(thread)arg;
    if(t==NULL) return;
    s=(session)(t->data);
    if(s==NULL) return;
    if(s->Con==NULL) return;

    log_debug(ZONE,"[main session] in session");
    s->connected=0;
    if(yahoo_connect(s->Con))
        s->connected=1;
    if(s->connected==1&&!yahoo_get_config(s->Con))
        s->connected=0;
    if(s->connected==0||yahoo_cmd_logon(s->Con,YAHOO_STATUS_AVAILABLE)) 
    {
        s->connected=0;
        yt_send_presence(s,NULL,jid_full(s->id),"unsubscribe","Failed to connect",NULL);
        yt_senderror(s->sendto,"502","Unable to connect to, or log in to Yahoo! server, Please check your username and password and try again	");
        log_error(ZONE,"FAILED GET CONFIG FOR (%s)[%s]",jid_full(s->id),jid_full(s->j));
    }
    else s->connected=1;
    /* 
    * hack: always send s10 requests when we log in,
    * if they are already subscribed, the server will
    * just automatically OK them.. ugly but uber effective
    */
    if(s->connected==1) 
    {
        pool p;
        xmlnode xres;
        int buddyn=0;
        struct yahoo_buddy *buddy=s->Con->buddies[buddyn];
        log_debug(ZONE,"we are a new user");
        xres=jid_xres(s->id);
        log_warn(ZONE,"[yahoo] calling subscribe for (%s)",jid_full(s->id));
        s->new_user=0;
        s->id=yt_subscribe_auth(s->sendto,s->Con->user,s->Con->password);
        p=pool_new();
        while(buddy!=NULL)
        {
            spool sp=spool_new(p);
            spooler(sp,buddy->id,"@",yt__host,sp);
            yt_send_presence(s,NULL,spool_print(sp),"subscribe","Old Yahoo! roster item",NULL);
            buddy=s->Con->buddies[++buddyn];
        }
        pool_free(p);
    }
    if(s->connected==1&&s->Con->sockfd>0)
    {
        s->stream=tstream_new(s->Con->sockfd,YAHOO_PAGER_HOST,&handle_read,&handle_err,(void*)s);
        log_warn(ZONE,"(%s) connected with yahoo: %s",jid_full(s->j),s->Con->user);
        yt_send_presence(s,NULL,jid_full(s->id),NULL,"Yahoo! Online",NULL);
        done=pth_event(PTH_EVENT_FUNC,yt_session_isdone,s,pth_time(0,100000));
        log_debug(ZONE,"(%s) waiting for KILL",jid_full(s->j));
        pth_wait(done);
    }
    yt_send_presence(s,NULL,jid_full(s->id),"unavailable","Yahoo! Offline",NULL);
    log_warn(ZONE, "Closing down session for %s",jid_full(s->j));
    if(s->stream!=NULL)
    {
        tstream_close(s->stream);
    }
    if(s->Con!=NULL) yahoo_free_context(s->Con);
    session_unlink(s);
    thread_free(t);	
    return;
}

/*
create the session data, and start the main loop
*/
thread yt_session_create(char *user,char *pword,jpacket jp,int new_user)
{
    thread t=NULL;
    session new=NULL;
    pth_attr_t attr;
        
    if(user==NULL||pword==NULL||jp==NULL)
    {
        /* umm. supply more info please! */
        log_error(ZONE,"ASSERT FAILED:not enough info to start session");
        return NULL;
    }
    log_debug(ZONE,"[session] New session for %s using %s, jid %s",user,pword,jid_full(jp->from));
    t=thread_new();
    if(t==NULL) return;
    new=pmalloco(t->p,sizeof(_session));
    new->j=jid_new(t->p,jid_full(jp->from));
    new->sendto=jid_new(t->p,jid_full(jp->from));
    yt_strip_jid(new->sendto);
    new->id=jid_new(t->p,jid_full(jp->to));
    new->t=t;
    new->stream=NULL;
    new->exit_flag=0;
    new->new_user=new_user;
    new->connected=0;
    new->prev=NULL;
    new->next=sessions;
    if(new->next!=NULL)
        new->next->prev=new;
    sessions=new;
    if((new->Con=yahoo_init(user,pword,&Opts))==NULL) return;
    new->status=YAHOO_STATUS_AVAILABLE;
    t->data=(void *)new;
    new->conferences=NULL;        
    attr=pth_attr_new();
    pth_attr_set(attr,PTH_ATTR_JOINABLE,FALSE);
    t->id=pth_spawn(attr,(void *)&yt_session_main,(void *)t);
    pth_attr_destroy(attr);
    log_debug(ZONE,"[session] We started a new session thread");
    return t;
}

